@@ -20,11 +20,14 @@
 #include <QQmlContext>
 #include <QStandardPaths>
 
+#include "contextMenuContext.h"
 #include "define.h"
+#include "dropperItem.h"
+#include "fileModel.h"
 #include "historyAction.h"
-#include "radialMap/item.h"
+#include "radialMap/map.h"
+#include "radialMap/radialMap.h"
 #include "scan.h"
-#include "settingsDialog.h"
 
 namespace Filelight
 {
@@ -47,7 +50,7 @@ MainContext::MainContext(QObject *parent)
     , m_histories(nullptr)
     , m_manager(new ScanManager(this))
 {
-    Config::read();
+    Config::instance()->read();
 
     auto engine = new QQmlApplicationEngine(this);
 
@@ -58,12 +61,48 @@ MainContext::MainContext(QObject *parent)
     l10nContext->setTranslationDomain(QStringLiteral(TRANSLATION_DOMAIN));
     engine->rootContext()->setContextObject(l10nContext);
 
+    qmlRegisterUncreatableMetaObject(Filelight::staticMetaObject, "org.kde.filelight", 1, 0, "Filelight", QStringLiteral("Access to enums & flags only"));
+
     auto about = new About(this);
     qRegisterMetaType<size_t>("size_t");
-    qmlRegisterType<RadialMap::Item>("org.kde.filelight", 1, 0, "RadialMapItem");
+    qmlRegisterType<DropperItem>("org.kde.filelight", 1, 0, "DropperItem");
     qmlRegisterSingletonInstance("org.kde.filelight", 1, 0, "About", about);
     qmlRegisterSingletonInstance("org.kde.filelight", 1, 0, "ScanManager", m_manager);
     qmlRegisterSingletonInstance("org.kde.filelight", 1, 0, "MainContext", this);
+    auto fileModel = new FileModel(this);
+    qmlRegisterSingletonInstance("org.kde.filelight", 1, 0, "FileModel", fileModel);
+
+    auto contextMenuContext = new ContextMenuContext(this);
+    qmlRegisterSingletonInstance("org.kde.filelight", 1, 0, "ContextMenuContext", contextMenuContext);
+    qmlRegisterUncreatableType<RadialMap::Segment>("org.kde.filelight", 1, 0, "Segment", QStringLiteral("only consumed, never created"));
+
+    // Do not initialize the map too early. It causes crashes on exit. Unclear why, probably a lifetime problem deep inside the retrofitted map. May be fixable
+    // with enough brain juice.
+    qmlRegisterSingletonType<RadialMap::Map>("org.kde.filelight", 1, 0, "RadialMap", [](QQmlEngine *engine, QJSEngine *scriptEngine) -> QObject * {
+        Q_UNUSED(engine)
+        Q_UNUSED(scriptEngine)
+        QQmlEngine::setObjectOwnership(RadialMap::Map::instance(), QQmlEngine::CppOwnership);
+        return RadialMap::Map::instance();
+    });
+    qmlRegisterSingletonType<Config>("org.kde.filelight", 1, 0, "Config", [](QQmlEngine *engine, QJSEngine *scriptEngine) -> QObject * {
+        Q_UNUSED(engine)
+        Q_UNUSED(scriptEngine)
+        QQmlEngine::setObjectOwnership(Config::instance(), QQmlEngine::CppOwnership);
+        return Config::instance();
+    });
+
+    connect(m_manager, &ScanManager::completed, RadialMap::Map::instance(), [](const auto &tree) {
+        if (tree) {
+            RadialMap::Map::instance()->make(tree);
+        }
+    });
+
+    connect(RadialMap::Map::instance(), &RadialMap::Map::signatureChanged, fileModel, [fileModel]() {
+        const auto tree = RadialMap::Map::instance()->root();
+        if (tree) {
+            fileModel->setTree(tree);
+        }
+    });
 
     engine->setInitialProperties({
         {QStringLiteral("inSandbox"),
@@ -244,16 +283,6 @@ void MainContext::setUrl(const QUrl &url)
     Q_EMIT urlChanged();
 }
 
-void MainContext::configFilelight() const
-{
-    auto dialog = new SettingsDialog(nullptr);
-
-    connect(dialog, &SettingsDialog::canvasIsDirty, this, &MainContext::canvasIsDirty);
-    connect(dialog, &SettingsDialog::mapIsInvalid, m_manager, &ScanManager::emptyCache);
-
-    dialog->show(); // deletes itself
-}
-
 bool MainContext::start(const QUrl &url) const
 {
     if (m_manager->running()) {
@@ -268,16 +297,6 @@ void MainContext::addHistoryAction(QObject *action)
     Q_EMIT historyActionsChanged();
 }
 
-void MainContext::connectMapItem(QObject *mapItem) const
-{ // we transfer shared_ptrs around, let's not do it via qml, we can't check if it is nullptr there
-    auto item = qobject_cast<RadialMap::Item *>(mapItem);
-    Q_ASSERT(item);
-    connect(m_manager, &ScanManager::completed, item, [item](const auto &tree) {
-        if (tree) {
-            item->create(tree);
-        }
-    });
-}
 } // namespace Filelight
 
 #include "mainContext.moc"
